require 'spec_helper'

class BankAccount
  attr_accessor :balance

  def initialize(initial_balance = 0)
    self.balance = initial_balance
  end

  def increase_balance(amount)
    self.balance += amount
  end

  def decrease_balance(amount)
    increase_balance(-amount)
  end
end

class MoneyTransferContext < Rietveld::Context
  role :source_account do
    def withdraw(amount)
      if balance >= amount
        decrease_balance(amount)
        amount
      else
        raise 'Insufficient funds'
      end
    end

    def transfer(amount)
      destination_account.deposit(withdraw(amount))
    end
  end

  role :destination_account do
    def deposit(amount)
      increase_balance(amount)
    end
  end

  def transfer(amount)
    source_account.transfer(amount)
  end
end


describe MoneyTransferContext do
  let(:source_account)      { BankAccount.new(200) }
  let(:destination_account) { BankAccount.new(100) }

  let(:context) { MoneyTransferContext.new(
                    source_account: source_account,
                    destination_account: destination_account) }

  context 'when transfering' do
    before { context.transfer(50) }

    it 'deposits given amount into destination account' do
      expect(destination_account.balance).to be(150)
    end

    it 'withdraws given amount from source account' do
      expect(source_account.balance).to be(150)
    end
  end
end
